# math_server.py
from mcp.server.fastmcp import FastMCP, Context

mcp = FastMCP("Math")

@mcp.tool()
def add(a: int, b: int) -> int:
    """Add two numbers"""
    return a + b

@mcp.tool()
def multiply(a: int, b: int) -> int:
    """Multiply two numbers"""
    return a * b

@mcp.prompt()
def summarize_request(text: str) -> str:
    """Generate a prompt asking for a summary."""
    return f"Please summarize the following text:\n\n{text}"

# Static resource
@mcp.resource("config://version")
def get_version(): 
    return "2.0.1"

# Dynamic resource template
@mcp.resource("users://{user_id}/profile")
def get_profile(user_id: int):
    # Fetch profile for user_id...
    return {"name": f"User {user_id}", "status": "active"}


@mcp.tool()
async def process_data(uri: str, ctx: Context):
    # Log a message to the client
    await ctx.info(f"Processing {uri}...")

    # Read a resource from the server
    data = await ctx.read_resource(uri)

    # Ask client LLM to summarize the data
    summary = await ctx.sample(f"Summarize: {data.content[:500]}")

    # Return the summary
    return summary.text


if __name__ == "__main__":
    # mcp.run(transport="stdio")
   mcp.run(transport="sse", host="0.0.0.0", port=8000)



from mcp.server.fastmcp import FastMCP, Context
from dotenv import load_dotenv
load_dotenv()
import os
from orm_bitrix24.entity import _Deal
from fast_bitrix24 import Bitrix
from typing import List
import asyncio
from pprint import pprint
from .userfields import get_all_info_fields
WEBHOOK=os.getenv("WEBHOOK")
bitrix=Bitrix(WEBHOOK)

class Deal(_Deal):
    pass
Deal.get_manager(bitrix)

mcp = FastMCP("bitrix24")


async def prepare_deal_fields_to_humman_format(fields: dict, all_info_fields: dict) -> dict:
    """
    Преобразует словарь с техническими ключами в словарь с человеческими названиями
    
    Args:
        fields: dict - словарь полей, например {'UF_CRM_1749724770090': '47', 'TITLE': 'тестовая сделка'}
        all_info_fields: dict - структура полей из get_all_info_fields
    
    Returns:
        dict - словарь с человеческими названиями, например {'этаж доставки': '1', 'Название': 'тестовая сделка'}
    """
    
    # Создаем маппинг: технический_ключ -> человеческое_название
    field_mapping = {}
    enumeration_values = {}  # Храним значения для полей типа enumeration
    
    # deal_fields = all_info_fields.get('deal', [])
    deal_fields=all_info_fields
    for field_info in deal_fields:
        for human_name, technical_info in field_info.items():
            # Извлекаем технический ключ из строки вида "TITLE (string)" или "UF_CRM_1749724770090 (enumeration):..."
            if '(' in technical_info:
                technical_key = technical_info.split(' (')[0]
                field_mapping[technical_key] = human_name
                
                # Если это поле типа enumeration, извлекаем значения
                if 'enumeration' in technical_info and ':\n' in technical_info:
                    values_part = technical_info.split(':\n', 1)[1]
                    enum_values = {}
                    for line in values_part.split(':\n'):
                        if '(ID: ' in line:
                            value_text = line.strip().split(' (ID: ')[0]
                            value_id = line.split('(ID: ')[1].split(')')[0]
                            enum_values[value_id] = value_text
                    enumeration_values[technical_key] = enum_values
    
    # Преобразуем входной словарь
    result = {}
    
    for tech_key, value in fields.items():
        # Получаем человеческое название
        human_name = field_mapping.get(tech_key, tech_key)
        
        # Если это поле enumeration и значение это ID, заменяем на текст
        if tech_key in enumeration_values and str(value) in enumeration_values[tech_key]:
            human_value = enumeration_values[tech_key][str(value)]
        else:
            human_value = value
            
        result[human_name] = human_value
    
    return result


@mcp.tool()
async def list_deal(filter_fields: dict[str,str]={}, fields_id: list[str]=["ID", "TITLE"]) -> dict:
    """Список сделок 
    filter_fields: dict[str, str] поля для фильтрации сделок 
    example:
    {
        "TITLE": "test"
        ">=DATE_CREATE": "2025-06-09"
        "<CLOSEDATE": "2025-06-11"
    }
    fields_id: list[str] id всех полей которые нужно получить (в том числе и из фильтра), если * то все поля
    """

    all_info_fields=await get_all_info_fields(['deal'], isText=False)
    all_info_fields=all_info_fields['deal']
    # pprint(all_info_fields)
    # 1/0
    prepare_deals=[]
    if '*' not in fields_id:     
        fields_id.append('ID')
        fields_id.append('TITLE')

    text=f'Список сделок по фильтру {filter_fields}:\n'
    deals : List[Deal] = await Deal.objects.filter(**filter_fields)
    if '*' in fields_id:
        for deal in deals:
            prepare_deals.append(deal._data)
    else:
        
        for deal in deals:
            prepare_deal={}
            for field in fields_id:
                if field in deal._data:
                    prepare_deal[field] = deal._data[field]
                else:
                    prepare_deal[field] = None
            prepare_deals.append(prepare_deal)

    for deal in prepare_deals:
        text+=f'=={deal["TITLE"]}==\n'
        pprint(deal)
        prepare_deal=await prepare_deal_fields_to_humman_format(deal, all_info_fields)
        for key, value in prepare_deal.items():
            text+=f'  {key}: {value}\n'
        text+='\n'
    return text

if __name__ == "__main__":
    # mcp.run(transport="sse", host="0.0.0.0", port=8000)
    # b=asyncio.run(list_deal(fields_id=['OPPORTUNITY']))
    b=asyncio.run(list_deal(fields_id=['*','UF_*'], filter_fields={">=DATE_CREATE": "2025-06-11"}))
    print(b)
    
    # Тест функции prepare_deal_fields_to_humman_format
    # async def test_prepare_fields():
    #     all_info_fields = await get_all_info_fields(['deal'], isText=False)
        
    #     # Тестовые данные
    #     test_fields = {
    #         'UF_CRM_1749724770090': '47',
    #         'TITLE': 'тестовая сделка',
    #         'OPPORTUNITY': '10000'
    #     }
        
    #     result = await prepare_deal_fields_to_humman_format(test_fields, all_info_fields)
    #     print("Исходные поля:", test_fields)
    #     print("Преобразованные поля:", result)
        
    # asyncio.run(test_prepare_fields())


# math_server.py
from fastmcp import FastMCP, Context
from .promts.promts import mcp as promts_mcp
from .resources.userfields import mcp as userfields_mcp_resource
from .tools.deal import mcp as deal_mcp
from .tools.userfields import mcp as userfields_mcp
from fastmcp.prompts.prompt import Message, PromptMessage, TextContent
from datetime import datetime
today=datetime.now().strftime("%Y-%m-%d %H:%M:%S")
# from fastmcp.server.auth import BearerAuthProvider
# from fastmcp.server.auth.providers.bearer import RSAKeyPair

# # Generate a new key pair
# key_pair = RSAKeyPair.generate()

# # Configure the auth provider with the public key
# auth = BearerAuthProvider(
#     public_key=key_pair.public_key,
#     issuer="https://dev.example.com",
#     audience="my-dev-server"
# )


# mcp = FastMCP("bitrix24", auth=auth)
mcp = FastMCP("bitrix24-main")


mcp.mount("userfields", userfields_mcp_resource, True)
mcp.mount("promts", promts_mcp, True)
mcp.mount("deal", deal_mcp, True)
mcp.mount('fields', userfields_mcp, True)


@mcp.prompt(description="главный промт для взаимодействия с сервером который нужно использовать каждый раз при взаимодействии с сервером")
def main_prompt() -> str:
    print('==========='*20)
    content= f"""
    Текущая дата: {today}
    при любом взаимодействии с сущностями сначало нужно получить все доступные поля сущности,
    используй get_all_info_fields. 
    если поле имеет тип enumeration, то значения полях это id значений поля а чтобы получить значение нужно использовать информацию из get_all_info_fields
    """
    # return PromptMessage(role="user", content=TextContent(type="text", text=content))
    return content

# mcp.mount("userfields", userfields_mcp, False)
# mcp.mount("promts", promts_mcp, False)
# mcp.mount("deal", deal_mcp, False)

# Generate a token for testing
# token = key_pair.create_token(
#     subject="dev-user",
#     issuer="https://dev.example.com",
#     audience="my-dev-server",
#     scopes=["read", "write"]
# )

# print(f"Test token: {token}")


if __name__ == "__main__":
    # mcp.run(transport="stdio")
   mcp.run(transport="sse", host="0.0.0.0", port=8000, timeout=600)



import asyncio
import os
import traceback
from fast_bitrix24 import Bitrix
from orm_bitrix24.entity import Portal
from pprint import pprint
from dotenv import load_dotenv
import json
load_dotenv()
# Инициализация клиента Bitrix24
webhook = os.getenv("WEBHOOK")
if not webhook:
    print("Необходимо установить переменную окружения WEBHOOK")
    exit()

from mcp.server.fastmcp import FastMCP, Context

mcp = FastMCP("userfields")





# async def get_all_info_fields(bitrix:Bitrix, entity:list[str]=['all']) -> str:
@mcp.tool()
async def get_all_info_fields(entity:list[str]=['all'], isText:bool=True) -> str:
    """
    Получение всех ID и названий полей сделки, контакта, компании
    args:
        bitrix: Bitrix - клиент Bitrix24
        entity: list[str] - ['deal', 'contact', 'company'] or ['all']
    return:
        allText: str - все ID и названия полей сделки, контакта, компании и также id значений полей типа enumeration
    """
    
    bitrix = Bitrix(webhook)
    # Инициализация Portal
    portal = Portal(bitrix)

    all_fields = {
        'deal': [],
        'contact': [],
        'company': []
    }

    if entity == ['all']:
        entity = ['deal', 'contact', 'company']

    for item in entity:
        if item == 'deal':
            fields = await portal.deal.fields.get_all()
        elif item == 'contact':
            fields = await portal.contact.fields.get_all()
        elif item == 'company':
            fields = await portal.company.fields.get_all()

        for field in fields:
            if field.type == 'enumeration':
                text=f'{field.name} ({field.type})'
                for value in field.values:
                    text+=f':\n  {value.value} (ID: {value.id})' if value.id else f'\n  {value.value}'
                all_fields[item].append({
                    field.title:text
                })
            else:
                all_fields[item].append({
                    field.title:f'{field.name} ({field.type})'
                })


    if entity == 'all':
        with open('bitrix_fields.json', 'w', encoding='utf-8') as f:
            json.dump(all_fields, f, indent=4, ensure_ascii=False)
    else:
        with open(f'bitrix_fields_{entity[0]}.json', 'w', encoding='utf-8') as f:
            json.dump(all_fields, f, indent=4, ensure_ascii=False)


    allText=''
    for key, value in all_fields.items():
        allText+=f'{key}:\n'
        for item in value:
            allText+=f'  {item}\n'
    
    if isText:
        return allText
    else:
        return all_fields





# if __name__ == "__main__":
    # asyncio.run(get_all_info_fields(bitrix, ['deal', 'company']))